#!/bin/sh
# ============================================================================
#
# This file is part of the 'SoftEther' package
# 
# This program is free software; you can redistribute it and/or modify it 
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# Author: Mijzelf <Mijzelf@live.com>
#
# ============================================================================
#

RELDIR=` dirname $0 `
PKG_NAME=Entware-ng
PKG_ROOT=/usr/local/zy-pkgs

HtmlHeader()
{
	echo Content-type: text/html
	echo 
}

DownloadLogfile()
{
	cat >&1 <<__END__
Pragma: public
Expires: 0
Cache-Control: must-revalidate, post-check=0, pre-check=0
Content-Type: application/force-download
Content-Type: application/octet-stream
Content-Type: application/download
Content-Disposition: attachment;filename="${logfile}"
Content-Transfer-Encoding: binary

__END__
	cat /opt/var/log/${1}
}

Chapter_Intro() 
{
	local title="Entware-ng ${ARCH}"
	if [ $1 -eq 0 ] ; then
		cat >&1 <<__END__
<h2><a href=pkgcgi.cgi?chapter=$2>${title}</a></h2>
__END__
		return
	fi
	
	cat >&1 <<__END__
<h2>${title}</h2>
This is a software repository for embedded devices like routers or network attached storages. About 1800 packages are available at moment of writing this article.
<br>
By now this repository is deprecated and no longer maintained. It's superseded by Entware.

__END__

}

Chapter_History()
{
	local title="History"
	if [ $1 -eq 0 ] ; then
		cat >&1 <<__END__
<h2><a href=pkgcgi.cgi?chapter=$2>${title}</a></h2>
__END__
		return
	fi
	
	cat >&1 <<__END__
<h2>${title}</h2>
It was founded as <a href=https://github.com/Entware/entware/>Entware project</a> to offer an alternative to very outdated <a href=https://en.wikipedia.org/wiki/Optware>Optware</a> packages. Within five years of development a several Entware forks was born to run on NASes, PCs and new ARM routers. Now x86, x64, MIPS, ARMv5 and ARMv7 forks are merged into Entware-ng. It's maintained by the same team as Entware before.
__END__
# '
}

Chapter_Shell()
{
	local title="Shell access"
	if [ $1 -eq 0 ] ; then
		cat >&1 <<__END__
<h2><a href=pkgcgi.cgi?chapter=$2>${title}</a></h2>
__END__
		return
	fi

	cat >&1 <<__END__
<h2>${title}</h2>
To use Entware-ng you need shell access. You'll need to enable a telnet or ssh service on the NAS for that. 
<h3>Firmware 5.10</h3>
You can do that on the desktop, in 'Control panel->Network->Terminal'.
<h3>Firmware 4.80</h3>
There is an installable package SSH.
<h3>Older</h3>
You can use the 3th party package '<a href=http://zyxel.nas-central.org/wiki/3rd_party_zypkgs#Tweaks>Tweaks</a>' to enable the telnet service, 
or <a href=http://zyxel.nas-central.org/wiki/3rd_party_zypkgs#Dropbear>Dropbear</a> to get an ssh server.
(You can even install <a href=http://zyxel.nas-central.org/wiki/3rd_party_zypkgs#anyterm>Anyterm</a>, which will give you a shell in a browser.)
<br><br>If you plan to install the Entware-ng ssh server, you can choose to use the <a href=http://zyxel.nas-central.org/wiki/Telnet_backdoor>Telnet Backdoor</a> to get one-off access.
<br><br>
<h3>Client side</h3>
On the client you need telnet or ssh client software. For Windows you can use <a href=http://www.chiark.greenend.org.uk/~sgtatham/putty/>PuTTY</a> for both purposes.
<br><br>Login as admin or root, using your admin password.
__END__
}	

Chapter_Repo()
{
	local title="Using repo"
	if [ $1 -eq 0 ] ; then
		cat >&1 <<__END__
<h2><a href=pkgcgi.cgi?chapter=$2>${title}</a></h2>
__END__
		return
	fi
	
	cat >&1 <<__END__
<h2>${title}</h2>
Entware-ng uses OPKG package manager, detailed help can be found <a href=https://wiki.openwrt.org/doc/techref/opkg>here</a>. Lets find transmission packages and install it's Web UI with dependencies, just for example:
<pre><code>
# opkg update
Downloading http://pkg.entware.net/binaries/${arch}/Packages.gz.
Updated list of available packages in /opt/var/opkg-lists/entware-ng.

# opkg list transmission*
transmission-cli - 2.84-4 - CLI utilities for transmission.
transmission-daemon - 2.84-4 - Transmission is a simple BitTorrent client.
 It features a very simple, intuitive interface
  on top on an efficient, cross-platform back-end.
   This package contains the daemon itself.
   transmission-remote - 2.84-4 - CLI remote interface for transmission.
   transmission-web - 2.84-4 - Webinterface resources for transmission.
   
# opkg install transmission-web
   Installing transmission-web (2.84-4) to root...
   Downloading http://pkg.entware.net/binaries/${arch}/transmission-web_2.84-4_${arch}.ipk.
   Installing transmission-daemon (2.84-4) to root...
   Downloading http://pkg.entware.net/binaries/${arch}/transmission-daemon_2.84-4_${arch}.ipk.
   ...
</code></pre>
<h3>More on the <a href=https://github.com/Entware-ng/Entware-ng/wiki>Wiki</a></h3>
__END__
# '
}

Chapter_OldKernel()
{
	[ -f /opt/sbin/kernel-2.6.24-support.sh ] || return

	local title="Old kernel support"
	if [ $1 -eq 0 ] ; then
		cat >&1 <<__END__
<h2><a href=pkgcgi.cgi?chapter=$2>${title}</a></h2>
__END__
		return
	fi

	cat >&1 <<__EOF__
<h2>${title}</h2>
On your box the old kernel patch is installed. Entware-ng targets to kernel 2.6.32 and newer, while your box runs $( uname -r ).
The old kernel patch has installed a libc which is compiled with support down to 2.6.24. The libc version is the same as the stock one, the only difference is a configure flag. 
I *think* the binaries which depend on libc won't care. At least I couldn't find any problems.<br><br>

When for some reason libc is reinstalled, the precious symlinks to the 2.6.24 version are broken, and you'll need to run
<pre><code>/opt/sbin/kernel-2.6.24-support.sh</code></pre>(as root) to restore it. This script is also run on each start of Entware-ng.
__EOF__
# '
}

Chapter_Packages()
{
	local title="Packages"
	if [ $1 -eq 0 ] ; then
		cat >&1 <<__END__
<h2><a href=pkgcgi.cgi?chapter=$2>${title}</a></h2>
__END__
		return
	fi
	
	cat >&1 <<__END__
<h2>${title}</h2>
<iframe src="http://pkg.entware.net/binaries/${ARCH}/Packages.html" height="600" width="99%"></iframe>
__END__
	
}

Chapter_FFP()
{
	local title="Entware-ng & FFP coexisting"
	if [ $1 -eq 0 ] ; then
		cat >&1 <<__END__
<h2><a href=pkgcgi.cgi?chapter=$2>${title}</a></h2>
__END__
		return
	fi
	
	cat >&1 <<__END__
<h2>${title}</h2>
FFP and Entware-ng are both designed to exist next to the firmware, without interfering. A side effect is that they can coexist fine. But there are some small problems.

It's all in the PATH. When you type a command, the shell searches it's PATH to find it, and execute it. You can examine your PATH by typing 
<pre><code>echo \$PATH</code></pre>Normally, it contains something like '/usr/bin:/usr/sbin:/bin:/sbin'. This means the shell will first look in /usr/bin for your command, then in /usr/sbin, then in /bin, ...<br>
Entware-ng puts itself in the PATH by prefixing it with '/opt/bin:/opt/sbin:', while FFP prefixes it with '/ffp/bin:/ffp/sbin:', but the way they do this is different.<br><br>
When logging in, your shell is read from /etc/passwd. By default this is /bin/sh. This shell executes /etc/profile to set it's environment. 
Entware-ng hooks this by making /etc/profile to source /opt/etc/profile, which adds the Entware directories to the PATH.<br><br>
The FFP package exchanges the default shell in /etc/passwd by it's own shell, /ffp/bin/sh, which doesn't use /etc/profile, but /ffp/etc/profile, which appends the FFP path.<br><br>

So if you have both FFP and Entware-ng installed, your PATH will by default not include the Entware-ng directories, as you are running an FFP shell, which doesn't use /etc/profile, 
and so doesn't use /opt/etc/profile either.<br><br>
This problem can be solved by providing the full path when you want to execute an Entware-ng command:<pre><code>/opt/bin/some_command</code></pre>
or you can source /opt/etc/profile:<pre><code>. /opt/etc/profile</code></pre>The latter has the problem that Entware-ng will put it's directories in front of the FFP directories, 
which means the Entware-ng binaries will be prefered above the FFP one's. (Which can be solved by providing the full path of the desired FFP binaries)
__END__
# '
}
	
Chapter_USBStick()
{
	local title="Moving to an USB stick"
	if [ $1 -eq 0 ] ; then
		cat >&1 <<__END__
<h2><a href=pkgcgi.cgi?chapter=$2>${title}</a></h2>
__END__
		return
	fi
	
	cat >&1 <<__END__
<h2>${title}</h2>
<h3>Redirecting</h3>
If you copy the contents of ${PKG_ROOT}/opt/ to an USB stick (or SD card) and create an (executable) script ${PKG_ROOT}/opt/redirect.sh which outputs the new root, 
Entware-ng will run from that new root.<br>
Example, copy to stick:<pre><code># cp -a ${PKG_ROOT}/opt /e-data/f8f5f38463c16267000c311ac176a098/</code></pre>The redirect script could be:<pre><code>#!/bin/sh

echo /e-data/f8f5f38463c16267000c311ac176a098/opt</code></pre>But as it is a script, you can also use blkid or something like that to find the mountpoint. If the returned value is not a directory, it's ignored.
<h3>Self containing stick</h3>
It's also possible to have a self containing stick, which starts itself. Download the latest zipfile <a href=http://downloads.zyxel.nas-central.org/Users/Mijzelf/Entware-ng/>here</a>, 
unzip it to a FAT formatted USB stick, plug it in the box, and boot it.
__END__
# '
}

Chapter_Logfile()
{
	local title="Logfile(s)"
	if [ $1 -eq 0 ] ; then
		cat >&1 <<__END__
<h2><a href=pkgcgi.cgi?chapter=$2>${title}</a></h2>
__END__
		return
	fi

	echo "<h2>${title}</h2>"

	local logfile=${PKG_ROOT}/opt/var/log/Entware-ng.log
	if [ -f $logfile ] ; then
		echo "<h3>Latest logfile</h3><p><pre><code><br>"
		cat $logfile
		echo "<br><br></code></pre></p>"
	fi
	
	local otherfiles=$( ls ${PKG_ROOT}/opt/var/log/ )
	if [ "${otherfiles}" != "Entware-ng.log" ] ; then
		echo "<h3>Other logs</h3><table><tr><th>Filename</th><th>Size</th><th>Time</th></tr>"
		for file in ${otherfiles}
		do
			[ -d "/opt/var/log/${file}" ] && continue
			line="$( ls -l "/opt/var/log/${file}" | rev )"
			local path=$( echo $line | cut -d ' ' -f 1 | rev )
			local time="$( echo $line | cut -d ' ' -f 2-4 | rev )"
			local size="$( echo $line | cut -d ' ' -f 5 | rev )"
			echo "<tr><td><a href=pkgcgi.cgi?logfile=${file}>${path}</a></td><td>${size}</td><td>${time}</td></tr>"
		done
		echo "</table><br><br>"
	fi
}

Chapter_ReleaseNotes()
{
	local title="Release notes"
	if [ $1 -eq 0 ] ; then
		cat >&1 <<__END__
<h2><a href=pkgcgi.cgi?chapter=$2>${title}</a></h2>
__END__
		return
	fi

	cat >&1 <<__EOF__
<h2>${title}</h2>
The changelog of Entware-ng itself can be found <a href=https://github.com/Entware-ng/Entware-ng/wiki/Changelog>here</a>.
__EOF__
	[ -f ${PKG_ROOT}/gui/${PKG_NAME}/RELEASENOTES ] || return
	cat >&1 <<__EOF__
<br><br>
	<h3>Release notes for the ZyXEL package</h3>
__EOF__
	echo "<ul>"
	cat ${PKG_ROOT}/gui/${PKG_NAME}/RELEASENOTES | while read line
	do
		[ "${line:0:1}" = "*" ] && echo "<li>${line:1}</li>" && continue
		[ "${line}" = "" ] && echo "<br>" && continue
		echo "</ul>${line}<ul>"		
	done
	echo "</ul>"
}

Index()
{
        . ${RELDIR}/cgi-bash.sh
	
	cgi_getvars GET ALL

	if [ "${logfile}" != "" ] ; then
		DownloadLogfile ${logfile}
		return
	fi
	

        ARCH=$( basename $(grep -v "^#" /opt/etc/opkg.conf | grep packages | awk '{print $3}' ) )

	HtmlHeader
	
	cat >&1 <<__END__
<html>
<head>
<style>
	body {
		background-color:MidnightBlue;
		color:lemonchiffon;
		
	}
	textarea {
		background-color:lemonchiffon;
		color:black;
	}
	input {
		background-color:MidnightBlue;
		color:lemonchiffon;
		font-size:140%;
	}
	a {
		color:Chartreuse;
	}
	p {
		border: 1px solid lemonchiffon;
	}
	iframe {
		background-color:lemonchiffon;
	}
</style>
</head>
<body>
__END__
	[ "${chapter}" = "" ] && chapter="Intro"
	
	for name in Intro History Shell OldKernel Repo Packages FFP USBStick Logfile ReleaseNotes
	do
		local active=0
		[ "${chapter}" = "${name}" ] && active=1
		
		Chapter_${name} $active $name
	done
	
	cat >&1 <<__END__
</body>
</html>
__END__
# '
}

Main()
{
	[ -z REQUEST_URI ] && NotFound && exit 0
	local uri=` echo ${REQUEST_URI} | cut -d '?' -f 1 `
	if echo ${uri} | grep "/pkg/${PKG_NAME}/pkgcgi.cgi" >/dev/null 
	then
	    Index && exit 0
	fi
}

if [ -f /tmp/${PKG_NAME}.log ] ; then
	exec 2>> /tmp/${PKG_NAME}.log
	echo "============================" >&2
	echo "` date `: $0 $@ ${REQUEST_URI}" >&2
	set >&2
	echo "============================" >&2
fi

Main
